#!/bin/bash


# Prints help

help()
{
  echo "ARGUMENTS :"
  echo "  1:  Nemo relative path                      |  default = Current dir"
  echo "  2:  Name of the configuration used          |  default = ORCA2(only OPA module no LIM)"
  echo "  3:  Name of the architecture used           |  default = Current dir"
  echo "  4:  Number of cores used                    |  default = 48"
  echo "  5:  Name of the cheduler used (slurm/lsf)   |  default = slurm"
  echo
}


# Checks if the paths given are correct.

Test_arguments()
{
  # Nemo path correct?
  if ! test -d "${Nemo_path}"; then
      echo "Nemo relative path: ${Nemo_path} is not found"
      exit 1
  fi

  # cfg exists?
  if [ $cfg != "default" ]; then
    if ! test -d "${Nemo_path}/cfgs/${cfg}"; then
      echo "configuration: ${cfg} doesn't exists in    ${Nemo_path}/cfgs   dir"
      exit 1
    fi
  fi

   # arch exists?
  if ! test -f "${Nemo_path}/arch/arch-${arch}.fcm"; then
      echo "architecture: arch-${arch}.fcm doesn't exists in ${Nemo_path}/arch dir"
      exit 1
  fi

  # scheduler correct?
  if [ $scheduler != "slurm" ] && [ $scheduler != "lsf" ]; then
      echo "$scheduler is not a valid scheduler"
      exit 1
  fi
}


# Checks if compiler flag -g is present in the arch file, this flag is needed for the correct creation of traces.
# If the flag is not present or the configuration used is the default one, Nemo is compilated again.


Test_Comp()
{
  line=$(sed -n '/^%FCFLAGS /p' $Nemo_path/arch/arch-$arch.fcm)
  if ! echo "${line}"|grep -q "\-g\b" || [ $cfg == "default" ]; then

    echo "-g flag not found in arch-${arch}.fcm creating new arch file to compile with extrae arch-${arch}_EXTRAE.fcm"
    echo

    cp ${Nemo_path}/arch/"arch-${arch}".fcm ${Nemo_path}/arch/arch-${arch}_EXTRAE.fcm
    sed -i '/%FCFLAGS/ s/$/ -g --finstrument-functions/' ${Nemo_path}/arch/arch-${arch}_EXTRAE.fcm

    if [ ${cfg} == "default" ] ; then

      python Job_Creator.py -f "compile" -j "compile" --set-core ${n_cores} -w "./makenemo -r ORCA2_ICE_PISCES -n ORCA2_EXTRAE -m ${arch} -d OCE del_key 'key_si3 key_top' -j48" -s $scheduler
    
    else 

      python Job_Creator.py -f "compile" -j "compile" --set-core ${n_cores} -w "./makenemo -r ${cfg} -n ${cfg}_EXTRAE -m ${arch}" -s $scheduler

    fi

    echo 'recompiling with -g flag'
    echo
    
    $job --wait compile.$scheduler

    if [ ${cfg} == "default" ] ; then

      cp ../NEMO/cfgs/ORCA2_EXTRAE/EXP00/* .
      sed -i '/_def_nemo-ice.xml\|def_nemo-pisces.xml/d' context_nemo.xml #DELETE ICE AND PISCES CONTEXT (NOT USED)
      mv weights_core_orca2_bicubic_noc.nc weights_core2_orca2_bicub.nc #RENAME WRONG NAMED FILES 
      mv weights_core_orca2_bilinear_noc.nc weights_core2_orca2_bilin.nc #RENAME WRONG NAMED FILES
      sed -i 's|ln_zdfiwm   = .true.|ln_zdfiwm   = .false.|g' namelist_cfg #CHANGE DUE TO NON EXISTING FILES

    else

      cp ../NEMO/cfgs/${cfg}_EXTRAE/EXP00/nemo .

    fi

  fi
}

Create_metrics()
{
  # Run nemo with extrae

  echo "Creating trace ..."
  python Job_Creator.py -f "run" -j "run" --set-core "48" -w "mpirun -np 48 ./trace.sh ./nemo" -s $scheduler
  $job --wait run.$scheduler

  # Cut trace to less than 1000 MiB 

  paramedir -c nemo.prv cutter_example.xml -o trace.prv
  echo "Trace information stored in trace.prv , trace.pcf and trace.row "
  echo "Creating metrics and storing theme in Metrics folder"

  # Creating folder

  mkdir Metrics
  cp trace.prv Metrics/
  cp trace.pcf Metrics/
  cp trace.row Metrics/
  cp Job_Creator.py Metrics/
  cd Metrics

  python Job_Creator.py -f "modelfactors" -j "modelfactors" --set-core "48" -w "modelfactors.py trace.prv" -s $scheduler
  $job --wait run.$scheduler
}



#Loads needed: intel/2019.4 or gcc  impi/2017.4 netcdf/4.4.1.1 hdf5/1.8.19 EXTRAE/3.7.1 perl Paraver Dinemas

#Default Init

Nemo_path="${1:-"."}"
cfg="${2:-"default"}"
arch="${3:-"X64_MN4"}"
n_cores="${4:-48}"
scheduler="${5:-"slurm"}"


if [ $scheduler == "slurm" ];then 

  job="sbatch"

else

  job="bsub"

fi

if [ $# -eq 0 ]; then

  echo "You entered no arguments, do you want to call help before continuing and try again?"

  select yn in "Yes" "No"; do

    case $yn in
        Yes ) 
        help 
        exit 1
        break
        ;;
        No ) 
        break
        ;;

    esac

  done
   
fi

Test_arguments
Test_Comp
Create_metrics
